@@ -8,14 +8,35 @@ RESPONSE = "response"
 
 ENTITIES = "entities"
 BILOU_ENTITIES = "bilou_entities"
+BILOU_ENTITIES_ROLE = "bilou_entities_role"
+BILOU_ENTITIES_GROUP = "bilou_entities_group"
 NO_ENTITY_TAG = "O"
 
+ENTITY_ATTRIBUTE_TYPE = "entity"
+ENTITY_ATTRIBUTE_GROUP = "group"
+ENTITY_ATTRIBUTE_ROLE = "role"
+ENTITY_ATTRIBUTE_VALUE = "value"
+ENTITY_ATTRIBUTE_TEXT = "text"
+ENTITY_ATTRIBUTE_START = "start"
+ENTITY_ATTRIBUTE_END = "end"
+ENTITY_ATTRIBUTE_CONFIDENCE = "confidence"
+ENTITY_ATTRIBUTE_CONFIDENCE_TYPE = (
+    f"{ENTITY_ATTRIBUTE_CONFIDENCE}_{ENTITY_ATTRIBUTE_TYPE}"
+)
+ENTITY_ATTRIBUTE_CONFIDENCE_GROUP = (
+    f"{ENTITY_ATTRIBUTE_CONFIDENCE}_{ENTITY_ATTRIBUTE_GROUP}"
+)
+ENTITY_ATTRIBUTE_CONFIDENCE_ROLE = (
+    f"{ENTITY_ATTRIBUTE_CONFIDENCE}_{ENTITY_ATTRIBUTE_ROLE}"
+)
+
 EXTRACTOR = "extractor"
 
 PRETRAINED_EXTRACTORS = {"DucklingHTTPExtractor", "SpacyEntityExtractor"}
 TRAINABLE_EXTRACTORS = {"MitieEntityExtractor", "CRFEntityExtractor", "DIETClassifier"}
 
 CLS_TOKEN = "__CLS__"
+POSITION_OF_CLS_TOKEN = -1
 
 MESSAGE_ATTRIBUTES = [TEXT, INTENT, RESPONSE]
 
