@@ -144,9 +144,8 @@ class RasaModelData:
         # update number of examples
         self.num_examples = self.number_of_examples()
 
-    def add_mask(self, key: Text, from_key: Text):
-        """Calculate mask for given key and put it under specified key."""
-
+    def add_lengths(self, key: Text, from_key: Text) -> None:
+        """Adds np.array of lengths of sequences to data under given key."""
         if not self.data.get(from_key):
             return
 
@@ -154,10 +153,8 @@ class RasaModelData:
 
         for data in self.data[from_key]:
             if data.size > 0:
-                # explicitly add last dimension to mask
-                # to track correctly dynamic sequences
-                mask = np.array([np.ones((x.shape[0], 1)) for x in data])
-                self.data[key].append(mask)
+                lengths = np.array([x.shape[0] for x in data])
+                self.data[key].append(lengths)
                 break
 
     def split(
